%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ beta, shat ] = MC_Inference_SqrtLASSO ( Y, X, lambda_p, q )

start_time = cputime;

[ nX pX ] = size(X);
[ nY p1 ] = size(Y) ;

if ( abs(nX-nY) > 0 )
    fpritnf('Mismatch on the dimensions of X and Y\n');
    beta = 0;
    shat = 0;
    return;
end

n = nX;
p = pX;

K.f = 0;
K.l = p + p;
K.q = 1+n;
if ( max(size(lambda_p)) ==  1)
    lambda = lambda_p * ones( p, 1 );
    vv = zeros(p,1);
    for j = 1 : p
        vv(j) = norm( X(:,j)/sqrt(n) );
    end
    lambda = lambda_p * vv;
    lambda(2:(q+1)) = 0;
else
    lambda = lambda_p;
end
%      beta_+   beta_-      t         
c = [ lambda/n ; lambda/n  ;    1/sqrt(n)   ; zeros(n,1) ];
b = [ Y ];
%     beta_+    beta_-      t           error
A = [  X         -X     zeros(n,1)     eye(n,n)  ];

pars.fid = 0;
pars.fid = 1;
pars.maxiter = 150;
pars.eps = 1.0e-8;
pars.bigeps = 1.0e-8;
pars.alg = 2;
pars.theta = 0.25;
pars.stepdif = 2;
pars.w = [1 1 ];
pars.numtol = 5 *1.0e-7;
pars.bignumtol = 0.9;
pars.numlvl = 0;
pars.denf =10;
pars.denq = 0.75;
pars.free = 1;
pars.sdp = 1;
pars.chol.skip = 1;
pars.chol.canceltol = 1.0e-12;
pars.chol.abstol = 1.0e-20;
pars.chol.maxuden = 500;
pars.cg.maxiter = 49;
pars.cg.refine = 1;
pars.cg.stagtol = 5*1.0e-14;
pars.cg.restol = 5*1.0e-3;
pars.cg.qprec = 1;
pars.vplot = 0;
pars.stopat = -1;
pars.errors = 0;


if ( 0 )
    [x,y,info] = sedumi(A,b,c,K,pars);
    opt_val_p = c'*x;
    opt_val_d = b'*y;

    beta = x(1:p)-x(p+1:2*p);
else
    OPTIONS.vers = 0;
    OPTIONS.gam = 0;
    OPTIONS.predcorr = 1;
    OPTIONS.expon = 1;
    OPTIONS.gaptol = 1.0000e-09;%11
    OPTIONS.inftol = 1.0000e-09;%11
    OPTIONS.steptol = 1.0000e-007;%9
    OPTIONS.maxit = 100;
    OPTIONS.printlevel = 3;
    OPTIONS.stoplevel = 1;
    OPTIONS.scale_data = 0;
    OPTIONS.spdensity = 0.4000;
    OPTIONS.rmdepconstr = 0;
    OPTIONS.cachesize = 128;
    OPTIONS.smallblkdim = 15;
    OPTIONS.parbarrier = [];
    OPTIONS.schurfun = [];
    OPTIONS.schurfun_par = [];
    
    [blk,At,C,b] = read_sedumi(A,b,c,K);
    [obj,XX,y,Z] = sqlp(blk,At,C,b,OPTIONS);

    opt_val_p = obj(1);
    opt_val_d = obj(2);

    betaOLD = XX{1}(1:p)-XX{1}(p+1:2*p);
    %for j = 1 : p
    %    %search for primal and dual evidence
    %    if ( abs(beta(j)) < 1.0000e-006  & min(Z{1}(j),Z{1}(j+500)) > 1.0000e-003  )
    %        beta(j) = 0;
    %    end
    %end
    
    for j = 1 : p
        %search for primal and dual evidence
        if ( XX{1}(j) < 1.0e-07  & Z{1}(j) > 0.250e-01 & XX{1}(j+p) < 1.0e-07 & Z{1}(j+p) > 0.250e-01  )
            XX{1}(j) = 0;
            XX{1}(j+p) = 0;            
        end
    end
    beta = XX{1}(1:p)-XX{1}(p+1:2*p);
end

shat = sum( ( abs(beta) > 0 ) );

if ( norm( (Y - X*beta)/sqrt(n) ) + lambda'*abs(beta)/n > norm( Y / sqrt(n) ) )
    %solution should be zero...
    beta = zeros(p,1);
    shat = 0;
end

total_time = cputime - start_time;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
