% % % Upload Volume_Inference_Growth.mat into MATLab
% % 
% %  X and Y are loaded
% % 

[ n p ] = size(X);
ALPHA = 0.1;
q=1; % number of variables of interest

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Lasso
k=1;
[ bb, bbint ] = regress(Y, X(:,[1 2]));
var_noise = mean((Y-X(:,[1 2])*bb).^2);
[ lambda ] = MC_PostLASSO_SimulateLambda ( X, sqrt(var_noise), k, 2000, n, ALPHA );
[ betaLASSO, sLASSO ] = MC_Inference_LASSO ( Y, X, lambda, q );                   
fprintf('Lasso with simple upper bound on sigma\n');

IND = [ 1 2 ];
for j = 3 : p	
    if ( abs(betaLASSO(j)) > 0.0000001 )
        IND = [ IND j ];
        fprintf('Equation Y on D, X: Variable: %s = %f\n',Label{j},betaLASSO(j));
    end       
end
[ bb, bbint ] = regress(Y, X(:,IND),0.1);
fprintf('Post-Lasso Estimate: %f, Naive C.I.  [ %f, %f ]\n',bb(2),bbint(2,1),bbint(2,2));


%%%%%%%%%%%%%%%%%%%%%%%%%%
[ lambda ] = MC_PostLASSO_SimulateLambdaSqrtLASSO( X, k, 2000, n, ALPHA );
gamma = zeros(p,1);
for j = 1 : p
    gamma(j) =  norm( X(:,j)/sqrt(n) );
end
gamma(1) = 0;
gamma(2) = 0;
[ betaSQLASSO, sSQLASSO] = SqrtLassoIterative(X, Y, lambda, gamma);
fprintf('Square root Lasso\n');
IND = [1 2];
for j = 3 : p	
    if ( abs(betaSQLASSO(j)) > 0.0000001 )
        IND = [ IND j ];
        fprintf('Equation Y on D, X: Variable: %s = %f\n',Label{j},betaSQLASSO(j));
    end       
end
[ bb, bbint ] = regress(Y, X(:,IND),0.1);
fprintf('Post-Sqrt-Lasso Estimate: %f, Naive C.I. [ %f, %f ]\n',bb(2),bbint(2,1),bbint(2,2));
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ beta2STEP, s2STEP, STDerror2STEP ] = MC_Inference_PostEstimator ( Y, X, betaLASSO, 0, q );
[ beta2STEPsq, s2STEPsq, STDerror_sq ]  = MC_Inference_PostEstimator ( Y, X, betaSQLASSO, 0, q );
                
        %%%%%%%
        %%% Double Selection
        %%% 1.  Run D on X, select X1
        %%% 2.  Run Y on X, select X2
        %%% 3.  Run Y on D, X1 and X2. 
%%%%    
psi = 1;
conf_lvl = 0.99;
[ betaLASSO_1, sLASSO_1] = MC_TE_LassoHeteroskedastic_unpenalized( X(:,2), [ X(:,1) X(:,1) X(:,3:p)], conf_lvl, psi, 5, [ 1 ] );
[ betaLASSO_2, sLASSO_2] = MC_TE_LassoHeteroskedastic_unpenalized(Y, [ X(:,1) X(:,1) X(:,3:p)],  conf_lvl, psi, 5, [ 1 ] );
                
IND_NEW = [];
for kk = 3 : p 
     if ( abs(betaLASSO_1(kk)) > 1.0e-8 || abs(betaLASSO_2(kk)) > 1.0e-8 )
                IND_NEW = [ IND_NEW kk ];
     end
end
NEWvecSECOND_9 = X(:, [ 1 2 IND_NEW ]);
[NEWpostSECOND_9, NEWpostINTSECOND_9] = regress(Y, NEWvecSECOND_9);
NEWmatSECOND_9 = inv(NEWvecSECOND_9'*NEWvecSECOND_9);
seNEWSECOND_9 = sqrt(NEWmatSECOND_9(2,2));
sNEWSECOND_9 = 2 + max(size(IND_NEW));

[ se_hetNEWSECOND_9 ] = Heteroskedastic_se ( Y, X(:,2), X(:,[ 1 IND_NEW]), NEWpostSECOND_9(2) );
  
fprintf('Double Selection\n');
for j = 3 : p	
    if ( abs(betaLASSO_1(j)) > 0.0000001 )
        fprintf('Equation D on X: Variable: %s = %f\n',Label{j},betaLASSO_1(j));
    end        
end
for j = 3 : p	
    if ( abs(betaLASSO_2(j)) > 0.0000001 )
        fprintf('Equation Y on X: Variable: %s = %f\n',Label{j},betaLASSO_2(j));
    end        
end
fprintf('Double Selection Estimate: %f (%f)\n',NEWpostSECOND_9(2),se_hetNEWSECOND_9);

%%%%%%
%% ALL VARIABLES
[ bb, bbint ] = regress(Y, X,0.1);
fprintf('L.S. with all covariates: %f, C.I. [ %f, %f ]\n',bb(2),bbint(2,1),bbint(2,2));
Ixx = inv(X'*X);
se_all  = sqrt(Ixx(2,2));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
ALL_StdErr = [ sqrt(n/(n-p))*norm((Y - X*bb)/sqrt(n),2)*se_all;
                sqrt(n/(n-sLASSO))*norm((Y - X*beta2STEP)/sqrt(n),2)*STDerror2STEP' ;
               sqrt(n/(n-sSQLASSO))*norm((Y - X*beta2STEPsq)/sqrt(n),2)*STDerror_sq';
               sqrt(n/(n-sNEWSECOND_9))*norm((Y - NEWvecSECOND_9*NEWpostSECOND_9)/sqrt(n),2)*seNEWSECOND_9];
ALL_ALPHA  = [ bb(2);
               beta2STEP(2:(q+1))';
               beta2STEPsq(2:(q+1))';  
               NEWpostSECOND_9(2)];
        
ALL_T = ALL_ALPHA ./ ALL_StdErr;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
%(1) all
%(2) post-LASSO
%(3) post-sqrt LASSO
%(4) Double Selection       
Method ={ 'All covariates', 'Post-Lasso', 'Post-Sqrt-Lasso', 'Double Selection'};
fprintf('Summary:\n');
for k=1:4
    fprintf('%s : %f (%f), t-value = %f \n',Method{k}, ALL_ALPHA(k),ALL_StdErr(k), ALL_T(k));
end